##
##   Copyright (C) 2006 Mario Theodoridis, mario@schmut.com
##
##   This "Original Work" is free; you can modify it under the terms of the
##   AFL Academic Free License. This "Original Work" is distributed in the
##   hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
##   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
##   See the AFL Academic Free License for more details. You should find a
##   copy of the AFL Academic Free License in the highest level directory of
##   this distribution; if not, you may obtain one at the Open Source
##   Initiative, http://www.opensource.org.
##

from xmltreewalker.printoperator import PrintOperator
from xml.sax.saxutils import quoteattr, escape 
from sys import stdout
from urlparse import urlparse
import os
import re

class CollateralPrintOperator(PrintOperator):
    """Print operator that resolves images.
    
    This will reprint the XML documnent walked over by walker.
    As it does so, it resolves and loads any referenced images and rewrites
    the urls to make then point to the loaded versions.
    All I/O is in unicode.
    """
    __name__ = 'CollateralPrintOperator'


    def __init__(self, baseDir, isActive = True, filterList=[], oStream=stdout, 
                 bubbles=True):
        '''Contstructor to specify output stream, on/off state and filters
        
        The base directory in which to look for collateral
        The output stream to write to
        The initial on/off state
        The filter list specifies when to toggle the on/off state
        '''
        # make sure basedir is a directory
        if not os.path.isdir(baseDir):
            raise IOError(u"'%s' is not a valid directory path" % baseDir)

        super(CollateralPrintOperator, self).__init__(isActive, filterList, 
                                                      oStream, bubbles)
        self.baseDir = baseDir
        self.subObjects = {}
        self.paths = {}


    def startHandler(self, name, attrs):
        if name == 'img':
            self.resolveAttrs(attrs)
        return PrintOperator.startHandler(self, name, attrs)


    def resolveAttrs(self, attrs):
        for i in range(0,len(attrs),2):
            if attrs[i] != 'src':
                continue
            path = attrs[i+1]
            if urlparse(path)[1]:
                # skip full paths
                continue
            if not os.path.isabs(path):
                path = os.path.join(self.baseDir, path)
            
            if not os.path.isfile(path):
                # skip non existent files
                continue
            
            if self.paths.has_key(path):
                # existing files only need src rewriting
                attrs[i+1] = self.paths[path]
                continue
            
            # handle the new file
            newsrc = os.path.basename(attrs[i+1])
            if self.subObjects.has_key(newsrc):
                # deal with duplicates due to them being in different 
                # directories with the same filename
                (base, ext) = os.path.splitext(newsrc)
                j = 2
                while self.subObjects.has_key(newsrc):
                    newsrc = "%s-%d%s" % (base, j, ext)
                    j += 1
                
            # fixme: this potential memory hog does not scale!!!
            self.subObjects[newsrc] = open(path, 'rb').read()
            self.paths[path] = newsrc
            attrs[i+1] = newsrc

