##
##   Copyright (C) 2006 Mario Theodoridis, mario@schmut.com
##
##   This "Original Work" is free; you can modify it under the terms of the
##   AFL Academic Free License. This "Original Work" is distributed in the
##   hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
##   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
##   See the AFL Academic Free License for more details. You should find a
##   copy of the AFL Academic Free License in the highest level directory of
##   this distribution; if not, you may obtain one at the Open Source
##   Initiative, http://www.opensource.org.
##



class Operator(object):
    """Base operator for XmlTreeWalker
    
    By default this will be inactive and do nothing when walked over by walker.
    It contains the base filtering logic. 
    Override any of the handlers to do something useful with the data.
    """

    
    def __init__(self, isActive = False, filterList=[], bubbles=True):
        '''Contstructor to on/off state and filters
        
        The initial on/off state
        The filter list specifies when to toggle the on/off state
        Whether this operator allows subsequent operators to process data.
        '''
        self.filters = filterList
        self.initialState = isActive
        self.resetState()
        self.bubbles = bubbles


    def __repr__(self):
        return 'initialState %s, isActive %s, %s' % \
               (self.initialState, self.isActive, repr(self.filters))


    def defaultHandler(self, data):
        return self.bubbles


    def procHandler(self, target, data):
        return self.bubbles


    def startHandler(self, name, attrs):
        return self.bubbles


    def dataHandler(self, data):
        return self.bubbles


    def endHandler(self, name):
        return self.bubbles


    def resetState(self):
        self.isActive = self.initialState


    def stateChanged(self, tagStack):
        '''Checks for state changed
        Returns true if the state has changed from initial to not initial due
        to filters. If the current state is not the initial state it returns 
        false as nothing changes.
        '''
        # already toggled
        if self.initialState != self.isActive: return False

        for filter in self.filters:
            if filter.test(tagStack): 
                self.isActive = not self.initialState
                return True

        return False



