##
##   Copyright (C) 2006 Mario Theodoridis, mario@schmut.com
##
##   This "Original Work" is free; you can modify it under the terms of the
##   AFL Academic Free License. This "Original Work" is distributed in the
##   hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
##   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
##   See the AFL Academic Free License for more details. You should find a
##   copy of the AFL Academic Free License in the highest level directory of
##   this distribution; if not, you may obtain one at the Open Source
##   Initiative, http://www.opensource.org.
##

from xmltreewalker.operator import Operator
from xml.sax.saxutils import quoteattr, escape 
from sys import stdout

class PrintOperator(Operator):
    """Default on print operator.
    
    By default this will reprint the XML documnent walked over by walker.
    This makes it an ideal default Operator for subtracting sections out 
    of documents.
    All I/O is in unicode.
    """
    __name__ = 'PrintOperator'


    def __init__(self, isActive = True, filterList=[], oStream = stdout, 
                 bubbles=True):
        '''Contstructor to specify output stream, on/off state and filters
        
        The initial on/off state
        The filter list specifies when to toggle the on/off state
        The output stream to write to
        '''
        super(PrintOperator, self).__init__(isActive, filterList, bubbles)
        self.out = oStream
        self.isOpened = False


    def defaultHandler(self, data):
        self.write(data)
        return self.bubbles


    def procHandler(self, target, data):
        if self.isOpened:
            self.write('>')
            self.isOpened = False
        self.write('<?%s %s?>' % (target, data))
        return self.bubbles


    def startHandler(self, name, attrs):
        if self.isOpened:
            self.write('>')
            self.isOpened = False
        self.write('<' + name + self.printAttrs(attrs))
        self.isOpened = True
        return self.bubbles


    def printAttrs(self, attrs):
        ret = '';
        for i in range(0,len(attrs),2):
            ret += ' %s=%s' % (attrs[i], quoteattr(attrs[i+1]))
        return ret


    def dataHandler(self, data):
        if self.isOpened:
            self.write('>')
            self.isOpened = False
        self.write(escape(data))
        return self.bubbles


    def endHandler(self, name):
        if self.isOpened:
            if name == 'a':
                # short closing anchor tags cause weird stuff on gecko :(
                self.write('></' + name + '>')
            else:
                self.write('/>')
        else:
            self.write('</' + name + '>')
        self.isOpened = False
        return self.bubbles


    def write(self, data):
        self.out.write(data)


