##
##   Copyright (C) 2006 Mario Theodoridis, mario@schmut.com
##
##   This "Original Work" is free; you can modify it under the terms of the
##   AFL Academic Free License. This "Original Work" is distributed in the
##   hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
##   implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
##   See the AFL Academic Free License for more details. You should find a
##   copy of the AFL Academic Free License in the highest level directory of
##   this distribution; if not, you may obtain one at the Open Source
##   Initiative, http://www.opensource.org.
##


class Tag:
    """Represents an XML tag and some extra context
    
    >>> tag = Tag('test')
    >>> tag
    test, {}, []
    >>> tag = Tag('test',['prop1' , '', 'prop2' , 'val2'])
    >>> tag
    test, {'prop1': '', 'prop2': 'val2'}, []
    >>> tag.append('operator')
    >>> tag
    test, {'prop1': '', 'prop2': 'val2'}, ['operator']
    >>> 
    """


    def __init__(self, name, attrList=[]):
        self.name = name
        self.attrMap = {}
        for i in range(0,len(attrList),2):
            self.attrMap[attrList[i]] = attrList[i+1]

        self.activeList = []


    def append(self, operator):
        """Adds operator to the active list of operators"""
        self.activeList.append(operator)


    def getAttribute(self, key):
        if not self.attrMap.has_key(key): return None
        return self.attrMap[key]


    def __repr__(self):
        return '%s, %s, %s' % (self.name, repr(self.attrMap), repr(self.activeList))


if __name__ == '__main__':
    tag1 = Tag('test1')
    print tag1
    tag2 = Tag('test2', ['prop1' , '', 'prop2' , 'val2'])
    tag2.append('operator')
    print tag2
    print 'tag1 prop1:%s' % tag1.getAttribute('prop1')
    print 'tag2 prop1:%s' % tag2.getAttribute('prop1')
    print 'tag2 prop2:%s' % tag2.getAttribute('prop2')
    import doctest
    doctest.testmod()

